@@ -5,36 +5,41 @@ module Debian.Debianize.BuildDependencies
     , debianBuildDepsIndep
     ) where
 
+
 import Control.Applicative ((<$>))
+import Control.Lens
 import Control.Monad.State (MonadState(get))
 import Control.Monad.Trans (MonadIO)
 import Data.Char (isSpace, toLower)
 import Data.Function (on)
-import Data.Lens.Lazy (access, getL)
-import Data.List as List (filter, map, minimumBy, nub)
+import Data.List as List (filter, groupBy, map, minimumBy, nub, sortBy)
 import Data.Map as Map (lookup, Map)
 import Data.Maybe (catMaybes, fromMaybe, isJust, isNothing)
-import Data.Set as Set (Set, member, toList, fromList, map, fold, union, empty)
+import Data.Set as Set (empty, fold, fromList, map, member, Set, singleton, toList, union)
 import Data.Version (showVersion, Version)
+import Debian.Debianize.BasicInfo (buildEnv, compilerFlavor, EnvSet(dependOS))
 import Debian.Debianize.Bundled (builtIn)
+import qualified Debian.Debianize.DebInfo as D
 import Debian.Debianize.DebianName (mkPkgName, mkPkgName')
-import Debian.Debianize.Monad as Monad (Atoms, DebT)
-import qualified Debian.Debianize.Types as T (buildDepends, buildDependsIndep, debianNameMap, epochMap, execMap, extraLibMap, missingDependencies, noDocumentationLibrary, noProfilingLibrary)
-import Debian.Debianize.Types.Atoms (EnvSet(dependOS), buildEnv, compilerFlavors)
-import qualified Debian.Debianize.Types.BinaryDebDescription as B (PackageType(Development, Documentation, Profiling))
+import Debian.Debianize.Monad as Monad (CabalInfo, CabalT)
+import qualified Debian.Debianize.BinaryDebDescription as B
+import qualified Debian.Debianize.CabalInfo as A
+import qualified Debian.Debianize.SourceDebDescription as S
 import Debian.Debianize.VersionSplits (packageRangesFromVersionSplits)
+import Debian.GHC (compilerPackageName)
 import Debian.Orphans ()
-import Debian.Relation (BinPkgName(..), Relation(..), Relations, checkVersionReq)
+import Debian.Relation (BinPkgName(..), checkVersionReq, Relation(..), Relations)
 import qualified Debian.Relation as D (BinPkgName(BinPkgName), Relation(..), Relations, VersionReq(EEQ, GRE, LTE, SGR, SLT))
 import Debian.Version (DebianVersion, parseDebianVersion)
 import Distribution.Compiler (CompilerFlavor(..))
-import Distribution.Package (Dependency(..), PackageIdentifier(..), PackageName(PackageName))
+import Distribution.Package (Dependency(..), PackageName(PackageName))
 import Distribution.PackageDescription (PackageDescription)
-import Distribution.PackageDescription as Cabal (allBuildInfo, BuildInfo(..), BuildInfo(buildTools, extraLibs, pkgconfigDepends), Executable(..))
-import qualified Distribution.PackageDescription as Cabal (PackageDescription(buildDepends, executables, package))
+import Distribution.PackageDescription as Cabal (allBuildInfo, BuildInfo(..), BuildInfo(buildTools, extraLibs, pkgconfigDepends), Executable(..), TestSuite(..))
+import qualified Distribution.PackageDescription as Cabal (PackageDescription(buildDepends, executables, testSuites))
 import Distribution.Version (anyVersion, asVersionIntervals, earlierVersion, foldVersionRange', fromVersionIntervals, intersectVersionRanges, isNoVersion, laterVersion, orEarlierVersion, orLaterVersion, toVersionIntervals, unionVersionRanges, VersionRange, withinVersion)
 import Distribution.Version.Invert (invertVersionRange)
 import Prelude hiding (init, log, map, unlines, unlines, writeFile)
+import System.Directory (findExecutable)
 import System.Exit (ExitCode(ExitSuccess))
 import System.IO.Unsafe (unsafePerformIO)
 import System.Process (readProcessWithExitCode)
@@ -46,13 +51,19 @@ data Dependency_
   | ExtraLibs Relations
     deriving (Eq, Show)
 
--- | In cabal a self dependency probably means the library is needed
--- while building the executables.  In debian it would mean that the
--- package needs an earlier version of itself to build, so we use this
--- to filter such dependencies out.
-selfDependency :: PackageIdentifier -> Dependency_ -> Bool
-selfDependency pkgId (BuildDepends (Dependency name _)) = name == pkgName pkgId
-selfDependency _ _ = False
+-- | Naive conversion of Cabal build dependencies to Debian
+-- dependencies will usually result in a self dependency, due to the
+-- fact that a Cabal executable often depends on the associated
+-- library to build.  Due to the fact that Debian build dependencies
+-- are global to the package, this results in unwanted self
+-- dependencies, which usually need to be filtered out.
+-- Unfortunately, some Debian packages actually do depend on an
+-- earlier version of themselves to build (e.g. most compilers.)  So a
+-- command line option is probably necessary.
+--
+-- selfDependency :: PackageIdentifier -> Dependency_ -> Bool
+-- selfDependency pkgId (BuildDepends (Dependency name _)) = name == pkgName pkgId
+-- selfDependency _ _ = False
 
 unboxDependency :: Dependency_ -> Maybe Dependency
 unboxDependency (BuildDepends d) = Just d
@@ -62,114 +73,115 @@ unboxDependency (ExtraLibs _) = Nothing -- Dependency (PackageName d) anyVersion
 
 -- |Debian packages don't have per binary package build dependencies,
 -- so we just gather them all up here.
-allBuildDepends :: Monad m => [Dependency] -> [Dependency] -> [Dependency] -> [String] -> DebT m [Dependency_]
-allBuildDepends buildDepends' buildTools' pkgconfigDepends' extraLibs' =
-    do atoms <- get
-       return $ nub $ List.map BuildDepends buildDepends' ++
-                      List.map BuildTools buildTools' ++
-                      List.map PkgConfigDepends pkgconfigDepends' ++
-                      [ExtraLibs (fixDeps atoms extraLibs')]
+allBuildDepends :: Monad m => PackageDescription -> CabalT m [Dependency_]
+allBuildDepends pkgDesc =
+    use (A.debInfo . D.enableTests) >>= \ testsEnabled ->
+    allBuildDepends'
+      (mergeCabalDependencies $
+       Cabal.buildDepends pkgDesc ++
+            concatMap (Cabal.targetBuildDepends . Cabal.buildInfo) (Cabal.executables pkgDesc) ++
+            (if testsEnabled then concatMap (Cabal.targetBuildDepends . Cabal.testBuildInfo) $ {-filter Cabal.testEnabled-} (Cabal.testSuites pkgDesc) else []))
+      (mergeCabalDependencies $ concatMap buildTools $ allBuildInfo pkgDesc)
+      (mergeCabalDependencies $ concatMap pkgconfigDepends $ allBuildInfo pkgDesc)
+      (concatMap extraLibs . allBuildInfo $ pkgDesc) >>=
+    return {- . List.filter (not . selfDependency (Cabal.package pkgDesc)) -}
     where
-      fixDeps :: Atoms -> [String] -> Relations
+      allBuildDepends' :: Monad m => [Dependency] -> [Dependency] -> [Dependency] -> [String] -> CabalT m [Dependency_]
+      allBuildDepends' buildDepends' buildTools' pkgconfigDepends' extraLibs' =
+          do atoms <- get
+             return $ nub $ List.map BuildDepends buildDepends' ++
+                            List.map BuildTools buildTools' ++
+                            List.map PkgConfigDepends pkgconfigDepends' ++
+                            [ExtraLibs (fixDeps atoms extraLibs')]
+
+      fixDeps :: CabalInfo -> [String] -> Relations
       fixDeps atoms xs =
           concatMap (\ cab -> fromMaybe [[D.Rel (D.BinPkgName ("lib" ++ List.map toLower cab ++ "-dev")) Nothing Nothing]]
-                                        (Map.lookup cab (getL T.extraLibMap atoms))) xs
+                                        (Map.lookup cab (view (A.debInfo . D.extraLibMap) atoms))) xs
+
+-- | Take the intersection of all the dependencies on a given package name
+mergeCabalDependencies :: [Dependency] -> [Dependency]
+mergeCabalDependencies =
+    List.map (foldl1 (\ (Dependency name range1) (Dependency _ range2) -> Dependency name (intersectVersionRanges range1 range2))) . groupBy ((==) `on` dependencyPackage) . sortBy (compare `on` dependencyPackage)
+    where
+      dependencyPackage (Dependency x _) = x
 
 -- The haskell-cdbs package contains the hlibrary.mk file with
 -- the rules for building haskell packages.
-debianBuildDeps :: (MonadIO m, Functor m) => PackageDescription -> DebT m D.Relations
+debianBuildDeps :: (MonadIO m, Functor m) => PackageDescription -> CabalT m D.Relations
 debianBuildDeps pkgDesc =
-    do hcs <- access compilerFlavors
+    do hc <- use (A.debInfo . D.flags . compilerFlavor)
+       let hcs = singleton hc -- vestigial
        let hcTypePairs =
                fold union empty $
-                  Set.map (\ hc -> Set.map (hc,) $ hcPackageTypes hc) hcs
-       cDeps <- cabalDeps hcTypePairs
-       bDeps <- access T.buildDepends
-       prof <- not <$> access T.noProfilingLibrary
-       let xs = nub $ [[D.Rel (D.BinPkgName "debhelper") (Just (D.GRE (parseDebianVersion ("7.0" :: String)))) Nothing],
-                       [D.Rel (D.BinPkgName "haskell-devscripts") (Just (D.GRE (parseDebianVersion ("0.8" :: String)))) Nothing],
+                  Set.map (\ hc' -> Set.map (hc',) $ hcPackageTypes hc') hcs
+       cDeps <- allBuildDepends pkgDesc >>= mapM (buildDependencies hcTypePairs) >>= return . {-nub .-} concat
+       bDeps <- use (A.debInfo . D.control . S.buildDepends)
+       prof <- not <$> use (A.debInfo . D.noProfilingLibrary)
+       official <- use (A.debInfo . D.official)
+       compat <- use (A.debInfo . D.compat)
+       let xs = nub $ [maybe [] (\ n -> [D.Rel (D.BinPkgName "debhelper") (Just (D.GRE (parseDebianVersion (show n)))) Nothing]) compat,
+                       [D.Rel (D.BinPkgName "haskell-devscripts") (Just $ D.GRE $ parseDebianVersion $ if official then "0.9" else "0.8" :: String) Nothing],
                        anyrel "cdbs"] ++
                       (if member GHC hcs
-                       then [anyrel "ghc"] ++ if prof
-                                              then [anyrel "ghc-prof"]
-                                              else []
+                       then [anyrel' (compilerPackageName GHC B.Development)] ++ if prof then [anyrel' (compilerPackageName GHC B.Profiling)] else []
                        else []) ++
-#if MIN_VERSION_Cabal(1,21,0)
+#if MIN_VERSION_Cabal(1,22,0)
                       (if member GHCJS hcs then [anyrel "ghcjs"] else []) ++
 #endif
                        bDeps ++
                        cDeps
        filterMissing xs
     where
-      cabalDeps hcTypePairs =
-          do deps <- allBuildDepends
-                          (Cabal.buildDepends pkgDesc ++ concatMap (Cabal.targetBuildDepends . Cabal.buildInfo) (Cabal.executables pkgDesc))
-                          (concatMap buildTools . allBuildInfo $ pkgDesc)
-                          (concatMap pkgconfigDepends . allBuildInfo $ pkgDesc)
-                          (concatMap extraLibs . allBuildInfo $ pkgDesc)
-             mapM (buildDependencies hcTypePairs) (List.filter (not . selfDependency (Cabal.package pkgDesc)) deps) >>= return . concat
       hcPackageTypes :: CompilerFlavor -> Set B.PackageType
       hcPackageTypes GHC = fromList [B.Development, B.Profiling]
-#if MIN_VERSION_Cabal(1,21,0)
+#if MIN_VERSION_Cabal(1,22,0)
       hcPackageTypes GHCJS = fromList [B.Development]
 #endif
       hcPackageTypes hc = error $ "Unsupported compiler flavor: " ++ show hc
 
 
-debianBuildDepsIndep :: (MonadIO m, Functor m) => PackageDescription -> DebT m D.Relations
+debianBuildDepsIndep :: (MonadIO m, Functor m) => PackageDescription -> CabalT m D.Relations
 debianBuildDepsIndep pkgDesc =
-    do hcs <- access compilerFlavors
-       doc <- get >>= return . not . getL T.noDocumentationLibrary
-       bDeps <- get >>= return . getL T.buildDependsIndep
-       cDeps <- cabalDeps
+    do hc <- use (A.debInfo . D.flags . compilerFlavor)
+       let hcs = singleton hc -- vestigial
+       doc <- not <$> use (A.debInfo . D.noDocumentationLibrary)
+       bDeps <- use (A.debInfo . D.control . S.buildDependsIndep)
+       cDeps <- allBuildDepends pkgDesc >>= mapM docDependencies
        let xs = nub $ if doc
-                      then (if member GHC hcs then [anyrel "ghc-doc"] else []) ++
-#if MIN_VERSION_Cabal(1,21,0)
+                      then (if member GHC hcs then [anyrel' (compilerPackageName GHC B.Documentation)] else []) ++
+#if MIN_VERSION_Cabal(1,22,0)
                            (if member GHCJS hcs then [anyrel "ghcjs"] else []) ++
 #endif
                            bDeps ++ concat cDeps
                       else []
        filterMissing xs
-    where
-      cabalDeps =
-          do deps <- allBuildDepends
-                           (Cabal.buildDepends pkgDesc) (concatMap buildTools . allBuildInfo $ pkgDesc)
-                           (concatMap pkgconfigDepends . allBuildInfo $ pkgDesc) (concatMap extraLibs . allBuildInfo $ pkgDesc)
-             let deps' = List.filter (not . selfDependency (Cabal.package pkgDesc)) deps
-             mapM docDependencies deps'
-{-
-      cabalDeps deb =
-          concat . List.map (\ x -> evalDebM (docDependencies x) deb)
-                     $ List.filter (not . selfDependency (Cabal.package pkgDesc))
-                     $ evalDebM
-                         (allBuildDepends
-                           (Cabal.buildDepends pkgDesc) (concatMap buildTools . allBuildInfo $ pkgDesc)
-                           (concatMap pkgconfigDepends . allBuildInfo $ pkgDesc) (concatMap extraLibs . allBuildInfo $ pkgDesc))
-                         deb
--}
 
 -- | The documentation dependencies for a package include the
 -- documentation package for any libraries which are build
--- dependencies, so we have access to all the cross references.
-docDependencies :: (MonadIO m, Functor m) => Dependency_ -> DebT m D.Relations
+-- dependencies, so we have use to all the cross references.
+docDependencies :: (MonadIO m, Functor m) => Dependency_ -> CabalT m D.Relations
 docDependencies (BuildDepends (Dependency name ranges)) =
-    do hcs <- access compilerFlavors
-       concat <$> mapM (\ hc -> dependencies hc B.Documentation name ranges) (toList hcs)
+    do hc <- use (A.debInfo . D.flags . compilerFlavor)
+       let hcs = singleton hc -- vestigial
+       omitProfDeps <- use (A.debInfo . D.omitProfVersionDeps)
+       concat <$> mapM (\ hc' -> dependencies hc' B.Documentation name ranges omitProfDeps) (toList hcs)
 docDependencies _ = return []
 
 -- | The Debian build dependencies for a package include the profiling
 -- libraries and the documentation packages, used for creating cross
 -- references.  Also the packages associated with extra libraries.
-buildDependencies :: (MonadIO m, Functor m) => Set (CompilerFlavor, B.PackageType) -> Dependency_ -> DebT m D.Relations
+buildDependencies :: (MonadIO m, Functor m) => Set (CompilerFlavor, B.PackageType) -> Dependency_ -> CabalT m D.Relations
 buildDependencies hcTypePairs (BuildDepends (Dependency name ranges)) =
-    concat <$> mapM (\ (hc, typ) -> dependencies hc typ name ranges) (toList hcTypePairs)
+    use (A.debInfo . D.omitProfVersionDeps) >>= \ omitProfDeps ->
+    concat <$> mapM (\ (hc, typ) -> dependencies hc typ name ranges omitProfDeps) (toList hcTypePairs)
 buildDependencies _ dep@(ExtraLibs _) =
-    do mp <- access T.execMap
+    do mp <- use (A.debInfo . D.execMap)
        return $ concat $ adapt mp dep
 buildDependencies _ dep =
     case unboxDependency dep of
       Just (Dependency _name _ranges) ->
-          do mp <- get >>= return . getL T.execMap
+          do mp <- get >>= return . view (A.debInfo . D.execMap)
              return $ concat $ adapt mp dep
       Nothing ->
           return []
@@ -182,18 +194,21 @@ adapt mp (BuildTools (Dependency (PackageName pkg) _)) =
 adapt _flags (ExtraLibs x) = [x]
 adapt _flags (BuildDepends (Dependency (PackageName pkg) _)) = [[[D.Rel (D.BinPkgName pkg) Nothing Nothing]]]
 
--- There are two reasons this may not work, or may work
+-- There are three reasons this may not work, or may work
 -- incorrectly: (1) the build environment may be a different
 -- distribution than the parent environment (the environment the
 -- autobuilder was run from), so the packages in that
--- environment might have different names, and (2) the package
+-- environment might have different names, (2) the package
 -- we are looking for may not be installed in the parent
--- environment.
+-- environment, and (3) the apt-file executable is not installed.
 aptFile :: String -> [Relations] -- Maybe would probably be more correct
-aptFile pkg =
-    unsafePerformIO $
-    do ret <- readProcessWithExitCode "apt-file" ["-l", "search", pkg ++ ".pc"] ""
-       return $ case ret of
+aptFile pkg = unsafePerformIO $
+    findExecutable "apt-file" >>= aptFile'
+  where
+    aptFile' Nothing = error "The apt-file executable could not be found."
+    aptFile' (Just aptfile) = do
+        ret <- readProcessWithExitCode aptfile ["-l", "search", pkg ++ ".pc"] ""
+        return $ case ret of
                   (ExitSuccess, out, _) ->
                       case takeWhile (not . isSpace) out of
                         "" -> error $ "Unable to locate a debian package containing the build tool " ++ pkg ++
@@ -211,15 +226,15 @@ anyrel' x = [D.Rel x Nothing Nothing]
 -- | Turn a cabal dependency into debian dependencies.  The result
 -- needs to correspond to a single debian package to be installed,
 -- so we will return just an OrRelation.
-dependencies :: MonadIO m => CompilerFlavor -> B.PackageType -> PackageName -> VersionRange -> DebT m Relations
-dependencies hc typ name cabalRange =
-    do atoms <- get
+dependencies :: MonadIO m => CompilerFlavor -> B.PackageType -> PackageName -> VersionRange -> Bool -> CabalT m Relations
+dependencies hc typ name cabalRange omitProfVersionDeps =
+    do nameMap <- use A.debianNameMap
        -- Compute a list of alternative debian dependencies for
        -- satisfying a cabal dependency.  The only caveat is that
        -- we may need to distribute any "and" dependencies implied
        -- by a version range over these "or" dependences.
        let alts :: [(BinPkgName, VersionRange)]
-           alts = case Map.lookup name (getL T.debianNameMap atoms) of
+           alts = case Map.lookup name nameMap of
                     -- If there are no splits for this package just
                     -- return the single dependency for the package.
                     Nothing -> [(mkPkgName hc name typ, cabalRange')]
@@ -256,8 +271,8 @@ dependencies hc typ name cabalRange =
             -- When we see a cabal equals dependency we need to turn it into
             -- a wildcard because the resulting debian version numbers have
             -- various suffixes added.
-      cabalRange' =
-          foldVersionRange'
+      cabalRange' | typ `elem` noVersionPackageType = anyVersion
+                  | otherwise = foldVersionRange'
             anyVersion
             withinVersion  -- <- Here we are turning equals into wildcard
             laterVersion
@@ -269,6 +284,7 @@ dependencies hc typ name cabalRange =
             intersectVersionRanges
             id
             cabalRange
+      noVersionPackageType = (if omitProfVersionDeps then [B.Profiling] else []) ++ [B.Documentation]
       simpler v1 v2 = minimumBy (compare `on` (length . asVersionIntervals)) [v1, v2]
       -- Simplify a VersionRange
       canon = fromVersionIntervals . toVersionIntervals
@@ -282,48 +298,44 @@ doBundled :: MonadIO m =>
           -> PackageName
           -> CompilerFlavor
           -> [D.Relation]
-          -> DebT m [D.Relation]
+          -> CabalT m [D.Relation]
 doBundled typ name hc rels =
     mapM doRel rels >>= return . concat
     where
       -- If a library is built into the compiler, this is the debian
       -- package name the compiler will conflict with.
       comp = D.Rel (compilerPackageName hc typ) Nothing Nothing
-      doRel :: MonadIO m => D.Relation -> DebT m [D.Relation]
+      doRel :: MonadIO m => D.Relation -> CabalT m [D.Relation]
       doRel rel@(D.Rel dname req _) = do
-        -- gver <- access ghcVersion
-        splits <- access T.debianNameMap
-        root <- access buildEnv >>= return . dependOS
+        -- gver <- use ghcVersion
+        splits <- use A.debianNameMap
+        root <- use (A.debInfo . D.flags . buildEnv) >>= return . dependOS
         -- Look at what version of the package is provided by the compiler.
         atoms <- get
         -- What version of this package (if any) does the compiler provide?
         let pver = maybe Nothing (Just . debianVersion'' atoms name) (builtIn splits hc root name)
         -- The name this library would have if it was in the compiler conflicts list.
         let naiveDebianName = mkPkgName hc name typ
-        return $ -- The compiler should appear in the build dependency
-                 -- if it provides a suitable version of the library,
-                 -- or if it conflicts with all versions of the
-                 -- library (which, if pver is Nothing, will certainly
-                 -- result in an error which needs to be corrected in
-                 -- the packaging.)
-                 (if isJust pver && (checkVersionReq req pver || (dname == naiveDebianName && conflictsWithHC naiveDebianName)) then [comp] else []) ++
-                 -- The library package can satisfy the dependency if
-                 -- the compiler doesn't provide a version, or if the
-                 -- compiler doesn't conflict with the package's
-                 -- debian name.
-                 (if isNothing pver || dname /= naiveDebianName || not (conflictsWithHC naiveDebianName) then [rel] else [])
-      compilerPackageName GHC B.Documentation = D.BinPkgName "ghc-doc"
-      compilerPackageName GHC B.Profiling = D.BinPkgName "ghc-prof"
-      compilerPackageName GHC B.Development = D.BinPkgName "ghc"
-      compilerPackageName GHC _ = D.BinPkgName "ghc" -- whatevs
-#if MIN_VERSION_Cabal(1,21,0)
-      compilerPackageName GHCJS B.Documentation = D.BinPkgName "ghcjs"
-      compilerPackageName GHCJS B.Profiling = error "Profiling not supported for GHCJS"
-      compilerPackageName GHCJS B.Development = D.BinPkgName "ghcjs"
-      compilerPackageName GHCJS _ = D.BinPkgName "ghcjs" -- whatevs
-#endif
-      compilerPackageName x _ = error $ "Unsupported compiler flavor: " ++ show x
-
+        -- The compiler should appear in the build dependency
+        -- if it provides a suitable version of the library,
+        -- or if it conflicts with all versions of the
+        -- library (which, if pver is Nothing, will certainly
+        -- result in an error which needs to be corrected in
+        -- the packaging.)
+        let compilerDependency = if isJust pver && (checkVersionReq req pver || (dname == naiveDebianName && conflictsWithHC naiveDebianName)) then [comp] else []
+        -- The library package can satisfy the dependency if
+        -- the compiler doesn't provide a version, or if the
+        -- compiler doesn't conflict with the package's
+        -- debian name.
+        let libraryDependency = if isNothing pver || dname /= naiveDebianName || not (conflictsWithHC naiveDebianName) then [rel] else []
+        -- Is the version number in the library dependency newer than
+        -- the compiler version?  If so it should appear to its left,
+        -- otherwise to its right.
+        return $ case req of
+                   Just (D.SLT lver) | Just lver < pver -> compilerDependency ++ libraryDependency
+                   Just (D.LTE lver) | Just lver < pver -> compilerDependency ++ libraryDependency
+                   Just (D.EEQ lver) | Just lver < pver -> compilerDependency ++ libraryDependency
+                   _ -> libraryDependency ++ compilerDependency
       -- FIXME: we are assuming here that ghc conflicts with all the
       -- library packages it provides but it no longer conflicts with
       -- libghc-cabal-dev.  We can now check these conflicts using the
@@ -338,23 +350,23 @@ doBundled :: MonadIO m =>
              B.PackageType  -- Documentation, Profiling, Development...
           -> PackageName    -- Cabal package name
           -> [D.Relation]   -- Original set of debian dependencies
-          -> DebT m [D.Relation] -- Modified debian dependencies accounting for the packages the compiler provides
+          -> CabalT m [D.Relation] -- Modified debian dependencies accounting for the packages the compiler provides
 doBundled hc typ name rels =
     concat <$> mapM doRel rels
     where
-      doRel :: MonadIO m => D.Relation -> DebT m [D.Relation]
+      doRel :: MonadIO m => D.Relation -> CabalT m [D.Relation]
       doRel rel@(D.Rel dname req _) = do
-        hc <- access
+        hc <- use
 -}
 
 -- Convert a cabal version to a debian version, adding an epoch number if requested
-debianVersion' :: Monad m => PackageName -> Version -> DebT m DebianVersion
+debianVersion' :: Monad m => PackageName -> Version -> CabalT m DebianVersion
 debianVersion' name v =
     do atoms <- get
-       return $ parseDebianVersion (maybe "" (\ n -> show n ++ ":") (Map.lookup name (getL T.epochMap atoms)) ++ showVersion v)
+       return $ parseDebianVersion (maybe "" (\ n -> show n ++ ":") (Map.lookup name (view A.epochMap atoms)) ++ showVersion v)
 
-debianVersion'' :: Atoms -> PackageName -> Version -> DebianVersion
-debianVersion'' atoms name v = parseDebianVersion (maybe "" (\ n -> show n ++ ":") (Map.lookup name (getL T.epochMap atoms)) ++ showVersion v)
+debianVersion'' :: CabalInfo -> PackageName -> Version -> DebianVersion
+debianVersion'' atoms name v = parseDebianVersion (maybe "" (\ n -> show n ++ ":") (Map.lookup name (view A.epochMap atoms)) ++ showVersion v)
 
 data Rels a = And {unAnd :: [Rels a]} | Or {unOr :: [Rels a]} | Rel' {unRel :: a} deriving Show
 
@@ -367,7 +379,7 @@ canonical (Rel' rel) = And [Or [Rel' rel]]
 canonical (And rels) = And $ concatMap (unAnd . canonical) rels
 canonical (Or rels) = And . List.map Or $ sequence $ List.map (concat . List.map unOr . unAnd . canonical) $ rels
 
-filterMissing :: Monad m => [[Relation]] -> DebT m [[Relation]]
+filterMissing :: Monad m => [[Relation]] -> CabalT m [[Relation]]
 filterMissing rels =
     get >>= \ atoms -> return $
-    List.filter (/= []) (List.map (List.filter (\ (Rel name _ _) -> not (Set.member name (getL T.missingDependencies atoms)))) rels)
+    List.filter (/= []) (List.map (List.filter (\ (Rel name _ _) -> not (Set.member name (view (A.debInfo . D.missingDependencies) atoms)))) rels)
