@@ -16,32 +16,29 @@ module Debian.Debianize.Goodies
     , serverAtoms
     , backupAtoms
     , execAtoms
-    , makeRulesHead
     ) where
 
-import Data.Char (toLower)
-import Data.Lens.Lazy (modL, access)
-import Data.List as List (map, intersperse, intercalate)
-import Data.Map as Map (insertWith)
+import Control.Lens
+import Data.Char (isSpace)
+import Data.List as List (dropWhileEnd, intercalate, intersperse, map)
+import Data.Map as Map (insert, insertWith)
 import Data.Maybe (fromMaybe)
-import Data.Monoid ((<>))
-import Data.Set as Set (insert, union, singleton, toList)
-import Data.Text as Text (Text, pack, unlines, intercalate)
-import Debian.Debianize.DebianName (debianNameBase)
-import Debian.Debianize.Monad (Atoms, DebT, execDebM)
-import Debian.Debianize.Prelude (trim, (%=), (+=), (++=), (+++=))
-import qualified Debian.Debianize.Types as T
-import qualified Debian.Debianize.Types.Atoms as T
-import qualified Debian.Debianize.Types.BinaryDebDescription as B
-import Debian.Debianize.VersionSplits (DebBase(DebBase))
+import Data.Monoid ((<>), mappend)
+import Data.Set as Set (insert, singleton, union)
+import Data.Text as Text (pack, Text, unlines)
+import qualified Debian.Debianize.DebInfo as D
+import Debian.Debianize.Monad (CabalInfo, CabalT, DebianT, execCabalM)
+import Debian.Debianize.Prelude (stripWith)
+import qualified Debian.Debianize.CabalInfo as A
+import qualified Debian.Debianize.BinaryDebDescription as B
 import Debian.Orphans ()
-import Debian.Pretty (Pretty(pretty))
-import Debian.Policy (apacheLogDirectory, apacheErrorLog, apacheAccessLog, databaseDirectory, serverAppLog, serverAccessLog)
+import Debian.Policy (apacheAccessLog, apacheErrorLog, apacheLogDirectory, databaseDirectory, dataDirectory, serverAccessLog, serverAppLog)
+import Debian.Pretty (ppShow, ppText)
 import Debian.Relation (BinPkgName(BinPkgName), Relation(Rel))
-import Distribution.Package (PackageIdentifier(..), PackageName(PackageName))
-import Distribution.PackageDescription as Cabal (PackageDescription(package, synopsis, description, author, maintainer, pkgUrl))
-import Distribution.Text (display)
-import Prelude hiding (writeFile, init, unlines, log, map)
+import Distribution.Package (PackageName(PackageName))
+import Distribution.PackageDescription as Cabal (PackageDescription(package, synopsis, description))
+import Distribution.Simple.Build.PathsModule (pkgPathEnvVar)
+import Prelude hiding (init, log, map, unlines, writeFile)
 import System.FilePath ((</>))
 
 showCommand :: String -> [String] -> String
@@ -59,10 +56,10 @@ translate str =
 -- to find out B's version number, version B.  Then write a rule into
 -- P's .substvar that makes P require that that exact version of A,
 -- and another that makes P conflict with any older version of A.
-tightDependencyFixup :: Monad m => [(BinPkgName, BinPkgName)] -> BinPkgName -> DebT m ()
+tightDependencyFixup :: Monad m => [(BinPkgName, BinPkgName)] -> BinPkgName -> DebianT m ()
 tightDependencyFixup [] _ = return ()
 tightDependencyFixup pairs p =
-    T.rulesFragments +=
+    D.rulesFragments %= Set.insert
           (Text.unlines $
                ([ "binary-fixup/" <> name <> "::"
                 , "\techo -n 'haskell:Depends=' >> debian/" <> name <> ".substvars" ] ++
@@ -75,33 +72,32 @@ tightDependencyFixup pairs p =
       equals (installed, dependent) = "\tdpkg-query -W -f='" <> display' dependent <> " (=$${Version})' " <>  display' installed <> " >> debian/" <> name <> ".substvars"
       newer  (installed, dependent) = "\tdpkg-query -W -f='" <> display' dependent <> " (>>$${Version})' " <> display' installed <> " >> debian/" <> name <> ".substvars"
       name = display' p
-      display' = pack . show . pretty
+      display' = ppText
 
 -- | Add a debian binary package to the debianization containing a cabal executable file.
-doExecutable :: Monad m => BinPkgName -> T.InstallFile -> DebT m ()
-doExecutable p f = T.executable ++= (p, f)
+doExecutable :: Monad m => BinPkgName -> D.InstallFile -> CabalT m ()
+doExecutable p f = (A.debInfo . D.executable) %= Map.insert p f
 
 -- | Add a debian binary package to the debianization containing a cabal executable file set up to be a server.
-doServer :: Monad m => BinPkgName -> T.Server -> DebT m ()
-doServer p s = T.serverInfo ++= (p, s)
+doServer :: Monad m => BinPkgName -> D.Server -> CabalT m ()
+doServer p s = (A.debInfo . D.serverInfo) %= Map.insert p s
 
 -- | Add a debian binary package to the debianization containing a cabal executable file set up to be a web site.
-doWebsite :: Monad m => BinPkgName -> T.Site -> DebT m ()
-doWebsite p w = T.website ++= (p, w)
+doWebsite :: Monad m => BinPkgName -> D.Site -> CabalT m ()
+doWebsite p w = (A.debInfo . D.website) %= Map.insert p w
 
 -- | Add a debian binary package to the debianization containing a cabal executable file set up to be a backup script.
-doBackups :: Monad m => BinPkgName -> String -> DebT m ()
+doBackups :: Monad m => BinPkgName -> String -> CabalT m ()
 doBackups bin s =
-    do T.backups ++= (bin, s)
-       T.depends bin %= (++ [[Rel (BinPkgName "anacron") Nothing Nothing]])
+    do (A.debInfo . D.backups) %= Map.insert bin s
+       (A.debInfo . D.binaryDebDescription bin . B.relations . B.depends) %= (++ [[Rel (BinPkgName "anacron") Nothing Nothing]])
        -- depends +++= (bin, Rel (BinPkgName "anacron") Nothing Nothing)
 
-describe :: Monad m => BinPkgName -> DebT m Text
-describe b =
-    do Just p <- access T.packageDescription
-       typ <- access (T.packageType b)
+describe :: Monad m => CabalT m Text
+describe =
+    do p <- use A.packageDescription
        return $
-          debianDescriptionBase p <> "\n" <>
+          debianDescriptionBase p {- <> "\n" <>
           case typ of
             Just B.Profiling ->
                 Text.intercalate "\n"
@@ -132,6 +128,7 @@ describe b =
                         [" .",
                          " Files associated with the " <> pack (display (pkgName (Cabal.package p))) <> " package."]
             -- x -> error $ "Unexpected library package name suffix: " ++ show x
+-}
 
 -- | The Cabal package has one synopsis and one description field
 -- for the entire package, while in a Debian package there is a
@@ -140,46 +137,46 @@ describe b =
 -- of the debian description, each of which is amended.
 debianDescriptionBase :: PackageDescription -> Text
 debianDescriptionBase p =
-    (pack . unwords . words $ Cabal.synopsis p) <>
-    case Cabal.description p of
-      "" -> ""
-      text ->
-          let text' = text ++ "\n" ++
-                      list "" ("\n Author: " ++) (Cabal.author p) ++
-                      list "" ("\n Upstream-Maintainer: " ++) (Cabal.maintainer p) ++
-                      list "" ("\n Url: " ++) (Cabal.pkgUrl p) in
-          "\n " <> (pack . trim . List.intercalate "\n " . List.map addDot . lines $ text')
+    pack $ List.intercalate "\n " $ (synop' : desc)
     where
-      addDot line = if all (flip elem " \t") line then "." else line
-      list :: b -> ([a] -> b) -> [a] -> b
-      list d f l = case l of [] -> d; _ -> f l
+      -- If we have a one line description and no synopsis, use
+      -- the description as the synopsis.
+      synop' = if null synop && length desc /= 1
+               then "WARNING: No synopsis available for package " ++ ppShow (package p)
+               else synop
+      synop :: String
+      -- I don't know why (unwords . words) was applied here.  Maybe I'll find out when
+      -- this version goes into production.  :-/  Ok, now I know, because sometimes the
+      -- short cabal description has more than one line.
+      synop = intercalate " " $ map (dropWhileEnd isSpace) $ lines $ Cabal.synopsis p
+      desc :: [String]
+      desc = List.map addDot . stripWith null $ map (dropWhileEnd isSpace) $ lines $ Cabal.description p
+      addDot line = if null line then "." else line
 
-oldClckwrksSiteFlags :: T.Site -> [String]
+oldClckwrksSiteFlags :: D.Site -> [String]
 oldClckwrksSiteFlags x =
     [ -- According to the happstack-server documentation this needs a trailing slash.
-      "--base-uri", "http://" ++ T.domain x ++ "/"
-    , "--http-port", show T.port]
-oldClckwrksServerFlags :: T.Server -> [String]
+      "--base-uri", "http://" ++ D.domain x ++ "/"
+    , "--http-port", show D.port]
+oldClckwrksServerFlags :: D.Server -> [String]
 oldClckwrksServerFlags x =
     [ -- According to the happstack-server documentation this needs a trailing slash.
-      "--base-uri", "http://" ++ T.hostname x ++ ":" ++ show (T.port x) ++ "/"
-    , "--http-port", show T.port]
+      "--base-uri", "http://" ++ D.hostname x ++ ":" ++ show (D.port x) ++ "/"
+    , "--http-port", show D.port]
 
 watchAtom :: PackageName -> Text
 watchAtom (PackageName pkgname) =
-    pack $ "version=3\nopts=\"downloadurlmangle=s|archive/([\\w\\d_-]+)/([\\d\\.]+)/|archive/$1/$2/$1-$2.tar.gz|,\\\nfilenamemangle=s|(.*)/$|" ++ pkgname ++
-           "-$1.tar.gz|\" \\\n    http://hackage.haskell.org/package/" ++ pkgname ++
-           " \\\n    ([\\d\\.]*\\d)/\n"
+    pack $ "version=3\nhttp://hackage.haskell.org/package/" ++ pkgname ++ "/distro-monitor .*-([0-9\\.]+)\\.(?:zip|tgz|tbz|txz|(?:tar\\.(?:gz|bz2|xz)))\n"
 
--- FIXME - use Atoms
-siteAtoms :: BinPkgName -> T.Site -> Atoms -> Atoms
-siteAtoms b site =
-    execDebM
-      (do T.installDir b "/etc/apache2/sites-available"
-          T.link b ("/etc/apache2/sites-available/" ++ T.domain site) ("/etc/apache2/sites-enabled/" ++ T.domain site)
-          T.file b ("/etc/apache2/sites-available" </> T.domain site) apacheConfig
-          T.installDir b (apacheLogDirectory b)
-          T.logrotateStanza +++= (b, singleton
+siteAtoms :: PackageDescription -> BinPkgName -> D.Site -> CabalInfo -> CabalInfo
+siteAtoms pkgDesc b site =
+    execCabalM
+      (do (A.debInfo . D.atomSet) %= (Set.insert $ D.InstallDir b "/etc/apache2/sites-available")
+          (A.debInfo . D.atomSet) %= (Set.insert $ D.Link b ("/etc/apache2/sites-available/" ++ D.domain site) ("/etc/apache2/sites-enabled/" ++ D.domain site))
+          (A.debInfo . D.atomSet) %= (Set.insert $ D.File b ("/etc/apache2/sites-available" </> D.domain site) apacheConfig)
+          (A.debInfo . D.atomSet) %= (Set.insert $ D.InstallDir b (apacheLogDirectory b))
+          (A.debInfo . D.logrotateStanza) %= Map.insertWith mappend b
+                              (singleton
                                    (Text.unlines $ [ pack (apacheAccessLog b) <> " {"
                                                    , "  copytruncate" -- hslogger doesn't notice when the log is rotated, maybe this will help
                                                    , "  weekly"
@@ -187,7 +184,8 @@ siteAtoms b site =
                                                    , "  compress"
                                                    , "  missingok"
                                                    , "}"]))
-          T.logrotateStanza +++= (b, singleton
+          (A.debInfo . D.logrotateStanza) %= Map.insertWith mappend b
+                              (singleton
                                    (Text.unlines $ [ pack (apacheErrorLog b) <> " {"
                                                    , "  copytruncate"
                                                    , "  weekly"
@@ -195,16 +193,16 @@ siteAtoms b site =
                                                    , "  compress"
                                                    , "  missingok"
                                                    , "}" ]))) .
-      serverAtoms b (T.server site) True
+      serverAtoms pkgDesc b (D.server site) True
     where
       -- An apache site configuration file.  This is installed via a line
       -- in debianFiles.
       apacheConfig =
           Text.unlines $
                    [  "<VirtualHost *:80>"
-                   , "    ServerAdmin " <> pack (T.serverAdmin site)
-                   , "    ServerName www." <> pack (T.domain site)
-                   , "    ServerAlias " <> pack (T.domain site)
+                   , "    ServerAdmin " <> pack (D.serverAdmin site)
+                   , "    ServerName www." <> pack (D.domain site)
+                   , "    ServerAlias " <> pack (D.domain site)
                    , ""
                    , "    ErrorLog " <> pack (apacheErrorLog b)
                    , "    CustomLog " <> pack (apacheAccessLog b) <> " combined"
@@ -233,34 +231,38 @@ siteAtoms b site =
                    , "    ProxyPass / http://127.0.0.1:" <> port' <> "/ nocanon"
                    , "    ProxyPassReverse / http://127.0.0.1:" <> port' <> "/"
                    , "</VirtualHost>" ]
-      port' = pack (show (T.port (T.server site)))
+      port' = pack (show (D.port (D.server site)))
 
--- FIXME - use Atoms
-serverAtoms :: BinPkgName -> T.Server -> Bool -> Atoms -> Atoms
-serverAtoms b server' isSite =
-    modL T.postInst (insertWith (\ old new -> if old /= new then error ("serverAtoms: " ++ show old ++ " -> " ++ show new) else old) b debianPostinst) .
-    modL T.installInit (Map.insertWith (\ old new -> if old /= new then error ("serverAtoms: " ++ show old ++ " -> " ++ show new) else old) b debianInit) .
+serverAtoms :: PackageDescription -> BinPkgName -> D.Server -> Bool -> CabalInfo -> CabalInfo
+serverAtoms pkgDesc b server' isSite =
+    over (A.debInfo . D.postInst) (insertWith failOnMismatch b debianPostinst) .
+    over (A.debInfo . D.installInit) (Map.insertWith failOnMismatch b debianInit) .
     serverLogrotate' b .
     execAtoms b exec
     where
-      exec = T.installFile server'
+      -- Combine two values (for insertWith) when there should only be
+      -- one.  If it happens twice with different values we should
+      -- really find out why.
+      failOnMismatch old new = if old /= new then error ("serverAtoms: " ++ show old ++ " -> " ++ show new) else old
+      exec = D.installFile server'
       debianInit =
           Text.unlines $
                    [ "#! /bin/sh -e"
                    , ""
                    , ". /lib/lsb/init-functions"
-                   , "test -f /etc/default/" <> pack (T.destName exec) <> " && . /etc/default/" <> pack (T.destName exec)
+                   , "test -f /etc/default/" <> pack (D.destName exec) <> " && . /etc/default/" <> pack (D.destName exec)
                    , ""
                    , "case \"$1\" in"
                    , "  start)"
-                   , "    test -x /usr/bin/" <> pack (T.destName exec) <> " || exit 0"
-                   , "    log_begin_msg \"Starting " <> pack (T.destName exec) <> "...\""
+                   , "    test -x /usr/bin/" <> pack (D.destName exec) <> " || exit 0"
+                   , "    log_begin_msg \"Starting " <> pack (D.destName exec) <> "...\""
                    , "    mkdir -p " <> pack (databaseDirectory b)
+                   , "    export " <> pack (pkgPathEnvVar pkgDesc "datadir") <> "=" <> pack (dataDirectory pkgDesc)
                    , "    " <> startCommand
                    , "    log_end_msg $?"
                    , "    ;;"
                    , "  stop)"
-                   , "    log_begin_msg \"Stopping " <> pack (T.destName exec) <> "...\""
+                   , "    log_begin_msg \"Stopping " <> pack (D.destName exec) <> "...\""
                    , "    " <> stopCommand
                    , "    log_end_msg $?"
                    , "    ;;"
@@ -270,11 +272,11 @@ serverAtoms b server' isSite =
                    , "esac"
                    , ""
                    , "exit 0" ]
-      startCommand = pack $ showCommand "start-stop-daemon" (startOptions ++ commonOptions ++ ["--"] ++ T.serverFlags server')
+      startCommand = pack $ showCommand "start-stop-daemon" (startOptions ++ commonOptions ++ ["--"] ++ D.serverFlags server')
       stopCommand = pack $ showCommand "start-stop-daemon" (stopOptions ++ commonOptions)
-      commonOptions = ["--pidfile", "/var/run/" ++ T.destName exec]
-      startOptions = ["--start", "-b", "--make-pidfile", "-d", databaseDirectory b, "--exec", "/usr/bin" </> T.destName exec]
-      stopOptions = ["--stop", "--oknodo"] ++ if T.retry server' /= "" then ["--retry=" ++ T.retry server' ] else []
+      commonOptions = ["--pidfile", "/var/run/" ++ D.destName exec]
+      startOptions = ["--start", "-b", "--make-pidfile", "-d", databaseDirectory b, "--exec", "/usr/bin" </> D.destName exec]
+      stopOptions = ["--stop", "--oknodo"] ++ if D.retry server' /= "" then ["--retry=" ++ D.retry server' ] else []
 
       debianPostinst =
           Text.unlines $
@@ -291,7 +293,7 @@ serverAtoms b server' isSite =
                           , "    service apache2 restart" ]
                      else []) ++
                     [ -- This gets done by the #DEBHELPER# code below.
-                      {- "    service " <> pack (show (pretty b)) <> " start", -}
+                      {- "    service " <> pack (show (pPrint b)) <> " start", -}
                       "    ;;"
                     , "esac"
                     , ""
@@ -301,26 +303,24 @@ serverAtoms b server' isSite =
 
 -- | A configuration file for the logrotate facility, installed via a line
 -- in debianFiles.
--- FIXME - use Atoms
-serverLogrotate' :: BinPkgName -> Atoms -> Atoms
+serverLogrotate' :: BinPkgName -> CabalInfo -> CabalInfo
 serverLogrotate' b =
-    modL T.logrotateStanza (insertWith Set.union b (singleton (Text.unlines $ [ pack (serverAccessLog b) <> " {"
+    over (A.debInfo . D.logrotateStanza) (insertWith Set.union b (singleton (Text.unlines $ [ pack (serverAccessLog b) <> " {"
                                  , "  weekly"
                                  , "  rotate 5"
                                  , "  compress"
                                  , "  missingok"
                                  , "}" ]))) .
-    modL T.logrotateStanza (insertWith Set.union b (singleton (Text.unlines $ [ pack (serverAppLog b) <> " {"
+    over (A.debInfo . D.logrotateStanza) (insertWith Set.union b (singleton (Text.unlines $ [ pack (serverAppLog b) <> " {"
                                  , "  weekly"
                                  , "  rotate 5"
                                  , "  compress"
                                  , "  missingok"
                                  , "}" ])))
 
--- FIXME - use Atoms
-backupAtoms :: BinPkgName -> String -> Atoms -> Atoms
+backupAtoms :: BinPkgName -> String -> CabalInfo -> CabalInfo
 backupAtoms b name =
-    modL T.postInst (insertWith (\ old new -> if old /= new then error $ "backupAtoms: " ++ show old ++ " -> " ++ show new else old) b
+    over (A.debInfo . D.postInst) (insertWith (\ old new -> if old /= new then error $ "backupAtoms: " ++ show old ++ " -> " ++ show new else old) b
                  (Text.unlines $
                   [ "#!/bin/sh"
                   , ""
@@ -329,52 +329,27 @@ backupAtoms b name =
                   , "    " <> pack ("/etc/cron.hourly" </> name) <> " --initialize"
                   , "    ;;"
                   , "esac" ])) .
-    execAtoms b (T.InstallFile { T.execName = name
-                               , T.destName = name
-                               , T.sourceDir = Nothing
-                               , T.destDir = Just "/etc/cron.hourly" })
+    execAtoms b (D.InstallFile { D.execName = name
+                               , D.destName = name
+                               , D.sourceDir = Nothing
+                               , D.destDir = Just "/etc/cron.hourly" })
 
--- FIXME - use Atoms
-execAtoms :: BinPkgName -> T.InstallFile -> Atoms -> Atoms
+execAtoms :: BinPkgName -> D.InstallFile -> CabalInfo -> CabalInfo
 execAtoms b ifile r =
-    modL T.rulesFragments (Set.insert (pack ("build" </> show (pretty b) ++ ":: build-ghc-stamp"))) .
+    over (A.debInfo . D.rulesFragments) (Set.insert (pack ("build" </> ppShow b ++ ":: build-ghc-stamp\n"))) .
     fileAtoms b ifile $
     r
 
--- FIXME - use Atoms
-fileAtoms :: BinPkgName -> T.InstallFile -> Atoms -> Atoms
+fileAtoms :: BinPkgName -> D.InstallFile -> CabalInfo -> CabalInfo
 fileAtoms b installFile' r =
-    fileAtoms' b (T.sourceDir installFile') (T.execName installFile') (T.destDir installFile') (T.destName installFile') r
+    fileAtoms' b (D.sourceDir installFile') (D.execName installFile') (D.destDir installFile') (D.destName installFile') r
 
--- FIXME - use Atoms
-fileAtoms' :: BinPkgName -> Maybe FilePath -> String -> Maybe FilePath -> String -> Atoms -> Atoms
+fileAtoms' :: BinPkgName -> Maybe FilePath -> String -> Maybe FilePath -> String -> CabalInfo -> CabalInfo
 fileAtoms' b sourceDir' execName' destDir' destName' r =
     case (sourceDir', execName' == destName') of
-      (Nothing, True) -> execDebM (T.installCabalExec b execName' d) r
-      (Just s, True) -> execDebM (T.install b (s </> execName') d) r
-      (Nothing, False) -> execDebM (T.installCabalExecTo b execName' (d </> destName')) r
-      (Just s, False) -> execDebM (T.installTo b (s </> execName') (d </> destName')) r
+      (Nothing, True) -> execCabalM ((A.debInfo . D.atomSet) %= (Set.insert $ D.InstallCabalExec b execName' d)) r
+      (Just s, True) -> execCabalM ((A.debInfo . D.atomSet) %= (Set.insert $ D.Install b (s </> execName') d)) r
+      (Nothing, False) -> execCabalM ((A.debInfo . D.atomSet) %= (Set.insert $ D.InstallCabalExecTo b execName' (d </> destName'))) r
+      (Just s, False) -> execCabalM ((A.debInfo . D.atomSet) %= (Set.insert $ D.InstallTo b (s </> execName') (d </> destName'))) r
     where
       d = fromMaybe "usr/bin" destDir'
-
--- | Build a suitable value for the head of the rules file.
-makeRulesHead :: (Monad m, Functor m) => DebT m Text
-makeRulesHead =
-    do DebBase b <- debianNameBase
-       compilers <- access T.compilerFlavors
-       let ls = ["DEB_CABAL_PACKAGE = " <> pack b] ++
-                -- If there is a single entry in the compiler list we
-                -- can tell debian/rules what compiler to use,
-                -- otherwise it has to figure this out from the
-                -- package names in debian/control.  This is required
-                -- to build a package without a library.
-                (case toList compilers of
-                   [x] -> ["DEB_DEFAULT_COMPILER = " <> pack (map toLower (show x))]
-                   _ -> []) ++
-                [""]
-       return $
-          Text.unlines $
-            ["#!/usr/bin/make -f", ""] ++
-            ls ++
-            ["include /usr/share/cdbs/1/rules/debhelper.mk",
-             "include /usr/share/cdbs/1/class/hlibrary.mk"]
