@@ -1,9 +1,10 @@
 -- | Code pulled out of cabal-debian that straightforwardly implements
 -- parts of the Debian policy manual, or other bits of Linux standards.
-{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, OverloadedStrings #-}
+{-# LANGUAGE CPP, DeriveDataTypeable, FlexibleInstances, OverloadedStrings #-}
 module Debian.Policy
     ( -- * Paths
       databaseDirectory
+    , dataDirectory
     , apacheLogDirectory
     , apacheErrorLog
     , apacheAccessLog
@@ -31,34 +32,52 @@ module Debian.Policy
     , Area(..)
     , parseUploaders
     , parseMaintainer
-    , getDebianMaintainer
+    , getCurrentDebianUser
     , haskellMaintainer
+    , License(..)
+    , fromCabalLicense
+    , toCabalLicense
+    , readLicense
     ) where
 
 import Codec.Binary.UTF8.String (decodeString)
 import Control.Arrow (second)
 import Control.Monad (mplus)
-import Data.Char (toLower, isSpace)
-import Data.List (groupBy, intercalate)
+import Data.Char (isSpace, toLower)
 import Data.Generics (Data, Typeable)
-import Data.Maybe (mapMaybe)
+import Data.List (groupBy, intercalate)
+import Data.Maybe (fromMaybe, mapMaybe)
 import Data.Monoid ((<>))
-import Data.Text (Text, pack, unpack, strip)
+import Data.Text (pack, strip, Text, unpack)
 import Debian.Debianize.Prelude (read')
-import Debian.Pretty (Pretty(pretty), text, empty)
+import Debian.Pretty (PP(..))
 import Debian.Relation (BinPkgName)
 import Debian.Version (DebianVersion, parseDebianVersion, version)
+import qualified Distribution.License as Cabal (License(..))
+import Distribution.Package (PackageIdentifier(pkgName))
+import Distribution.PackageDescription (PackageDescription(package))
+import Distribution.Text (display)
 import System.Environment (getEnvironment)
 import System.FilePath ((</>))
 import System.Process (readProcess)
 import Text.Parsec (parse)
-import Text.ParserCombinators.Parsec.Rfc2822 (NameAddr(..), address)
+import Text.ParserCombinators.Parsec.Rfc2822 (address, NameAddr(..))
+import Text.PrettyPrint.HughesPJClass (Pretty(pPrint), text)
+import Text.Read (readMaybe)
 
 databaseDirectory :: BinPkgName -> String
-databaseDirectory x = "/srv" </> show (pretty x)
+databaseDirectory x = "/srv" </> show (pPrint . PP $ x)
+
+dataDirectory :: PackageDescription -> String
+dataDirectory pkgDesc = "/usr/share" </> showPkgName (pkgName (package pkgDesc))
+    where
+      -- Copied from Distribution.Simple.Build.PatsModule in Cabal
+      showPkgName = map fixchar . display
+      fixchar '-' = '_'
+      fixchar c   = c
 
 apacheLogDirectory :: BinPkgName -> String
-apacheLogDirectory x =  "/var/log/apache2/" ++ show (pretty x)
+apacheLogDirectory x =  "/var/log/apache2/" ++ show (pPrint . PP $ x)
 
 apacheErrorLog :: BinPkgName -> String
 apacheErrorLog x = apacheLogDirectory x </> errorLogBaseName
@@ -67,7 +86,7 @@ apacheAccessLog :: BinPkgName -> String
 apacheAccessLog x = apacheLogDirectory x </> accessLogBaseName
 
 serverLogDirectory :: BinPkgName -> String
-serverLogDirectory x = "/var/log/" ++ show (pretty x)
+serverLogDirectory x = "/var/log/" ++ show (pPrint . PP $ x)
 
 serverAppLog :: BinPkgName -> String
 serverAppLog x = serverLogDirectory x </> appLogBaseName
@@ -104,9 +123,9 @@ getDebhelperCompatLevel =
 
 data StandardsVersion = StandardsVersion Int Int Int (Maybe Int) deriving (Eq, Ord, Show, Data, Typeable)
 
-instance Pretty StandardsVersion where
-    pretty (StandardsVersion a b c (Just d)) = pretty (show a) <> text "." <> pretty (show b) <> text "." <> pretty (show c) <> text "." <> pretty (show d)
-    pretty (StandardsVersion a b c Nothing) = pretty (show a) <> text "." <> pretty (show b) <> text "." <> pretty (show c)
+instance Pretty (PP StandardsVersion) where
+    pPrint (PP (StandardsVersion a b c (Just d))) = text (show a) <> text "." <> text (show b) <> text "." <> text (show c) <> text "." <> text (show d)
+    pPrint (PP (StandardsVersion a b c Nothing)) = text (show a) <> text "." <> text (show b) <> text "." <> text (show c)
 
 -- | Assumes debian-policy is installed
 getDebianStandardsVersion :: IO (Maybe StandardsVersion)
@@ -129,9 +148,9 @@ data SourceFormat
     | Quilt3
     deriving (Eq, Ord, Show, Data, Typeable)
 
-instance Pretty SourceFormat where
-    pretty Quilt3 = text "3.0 (quilt)\n"
-    pretty Native3 = text "3.0 (native)\n"
+instance Pretty (PP SourceFormat) where
+    pPrint (PP Quilt3) = text "3.0 (quilt)\n"
+    pPrint (PP Native3) = text "3.0 (native)\n"
 
 readSourceFormat :: Text -> Either Text SourceFormat
 readSourceFormat s =
@@ -158,8 +177,8 @@ readPriority s =
       "extra" -> Extra
       x -> error $ "Invalid priority string: " ++ show x
 
-instance Pretty PackagePriority where
-    pretty = pretty . map toLower . show
+instance Pretty (PP PackagePriority) where
+    pPrint = text . map toLower . show . unPP
 
 -- | The architectures for which a binary deb can be built.
 data PackageArchitectures
@@ -168,14 +187,10 @@ data PackageArchitectures
     | Names [String] -- ^ The list of suitable architectures
     deriving (Read, Eq, Ord, Show, Data, Typeable)
 
-instance Pretty PackageArchitectures where
-    pretty All = text "all"
-    pretty Any = text "any"
-    pretty (Names xs) = pretty $ intercalate " " xs
-
-instance Pretty (Maybe PackageArchitectures) where
-    pretty Nothing = empty
-    pretty (Just x) = pretty x
+instance Pretty (PP PackageArchitectures) where
+    pPrint (PP All) = text "all"
+    pPrint (PP Any) = text "any"
+    pPrint (PP (Names xs)) = text $ intercalate " " xs
 
 parsePackageArchitectures :: String -> PackageArchitectures
 parsePackageArchitectures "all" = All
@@ -196,9 +211,9 @@ readSection s =
       (a, '/' : _) -> error $ "readSection - unknown area: " ++ show a
       (a, _) -> MainSection a
 
-instance Pretty Section where
-    pretty (MainSection sec) = pretty sec
-    pretty (AreaSection area sec) = pretty area <> text "/" <> pretty sec
+instance Pretty (PP Section) where
+    pPrint (PP (MainSection sec)) = text sec
+    pPrint (PP (AreaSection area sec)) = pPrint (PP area) <> text "/" <> text sec
 
 -- Is this really all that is allowed here?  Doesn't Ubuntu have different areas?
 data Area
@@ -207,10 +222,10 @@ data Area
     | NonFree
     deriving (Read, Eq, Ord, Show, Data, Typeable)
 
-instance Pretty Area where
-    pretty Main = text "main"
-    pretty Contrib = text "contrib"
-    pretty NonFree = text "non-free"
+instance Pretty (PP Area) where
+    pPrint (PP Main) = text "main"
+    pPrint (PP Contrib) = text "contrib"
+    pPrint (PP NonFree) = text "non-free"
 
 {-
 Create a debian maintainer field from the environment variables:
@@ -237,8 +252,8 @@ now. Here is what the man page for dch has to say:
  DEBEMAIL and DEBFULLNAME when using this script.
 
 -}
-getDebianMaintainer :: IO (Maybe NameAddr)
-getDebianMaintainer =
+getCurrentDebianUser :: IO (Maybe NameAddr)
+getCurrentDebianUser =
     do env <- map (second decodeString) `fmap` getEnvironment
        return $ do fullname <- lookup "DEBFULLNAME" env `mplus` lookup "NAME" env
                    email    <- lookup "DEBEMAIL" env `mplus` lookup "EMAIL" env
@@ -276,3 +291,122 @@ parseMaintainer x =
       Right [y] -> Right y
       Right [] -> Left $ "Missing maintainer: " ++ show x
       Right ys -> Left $ "Too many maintainers: " ++ show ys
+
+-- | Official Debian license types as described in
+-- <https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/#license-specification>.
+data License
+    = Public_Domain     -- ^ No license required for any purpose; the work is not subject to copyright in any jurisdiction.
+    | Apache            -- ^ Apache license 1.0, 2.0.
+    | Artistic          -- ^ Artistic license 1.0, 2.0.
+    | BSD_2_Clause      -- ^ Berkeley software distribution license, 2-clause version.
+    | BSD_3_Clause      -- ^ Berkeley software distribution license, 3-clause version.
+    | BSD_4_Clause      -- ^ Berkeley software distribution license, 4-clause version.
+    | ISC               -- ^ Internet Software Consortium, sometimes also known as the OpenBSD License.
+    | CC_BY             -- ^ Creative Commons Attribution license 1.0, 2.0, 2.5, 3.0.
+    | CC_BY_SA          -- ^ Creative Commons Attribution Share Alike license 1.0, 2.0, 2.5, 3.0.
+    | CC_BY_ND          -- ^ Creative Commons Attribution No Derivatives license 1.0, 2.0, 2.5, 3.0.
+    | CC_BY_NC          -- ^ Creative Commons Attribution Non-Commercial license 1.0, 2.0, 2.5, 3.0.
+    | CC_BY_NC_SA       -- ^ Creative Commons Attribution Non-Commercial Share Alike license 1.0, 2.0, 2.5, 3.0.
+    | CC_BY_NC_ND       -- ^ Creative Commons Attribution Non-Commercial No Derivatives license 1.0, 2.0, 2.5, 3.0.
+    | CC0               -- ^ Creative Commons Zero 1.0 Universal. Omit "Universal" from the license version when forming the short name.
+    | CDDL              -- ^ Common Development and Distribution License 1.0.
+    | CPL               -- ^ IBM Common Public License.
+    | EFL               -- ^ The Eiffel Forum License 1.0, 2.0.
+    | Expat             -- ^ The Expat license.
+    | GPL               -- ^ GNU General Public License 1.0, 2.0, 3.0.
+    | LGPL              -- ^ GNU Lesser General Public License 2.1, 3.0, or GNU Library General Public License 2.0.
+    | GFDL              -- ^ GNU Free Documentation License 1.0, 1.1, 1.2, or 1.3. Use GFDL-NIV instead if there are no Front-Cover or Back-Cover Texts or Invariant Sections.
+    | GFDL_NIV          -- ^ GNU Free Documentation License, with no Front-Cover or Back-Cover Texts or Invariant Sections. Use the same version numbers as GFDL.
+    | LPPL              -- ^ LaTeX Project Public License 1.0, 1.1, 1.2, 1.3c.
+    | MPL               -- ^ Mozilla Public License 1.1.
+    | Perl              -- ^ erl license (use "GPL-1+ or Artistic-1" instead)
+    | Python            -- ^ Python license 2.0.
+    | QPL               -- ^ Q Public License 1.0.
+    | W3C               -- ^ W3C Software License For more information, consult the W3C Intellectual Rights FAQ.
+    | Zlib              -- ^ zlib/libpng license.
+    | Zope              -- ^ Zope Public License 1.0, 1.1, 2.0, 2.1.
+    | OtherLicense String
+                        -- ^ A license name associated with the subsequent text of the License: field or in
+                        -- a Files paragraph of the same debian/copyright file, or in a License: paragraph.
+    deriving (Read, Show, Eq, Ord, Data, Typeable)
+
+-- We need a license parse function that converts these strings back
+-- into License values.
+instance Pretty License where
+    pPrint Public_Domain = text "public-domain"
+    pPrint Apache = text "Apache"
+    pPrint Artistic = text "Artistic"
+    pPrint BSD_2_Clause = text "BSD2"
+    pPrint BSD_3_Clause = text "BSD3"
+    pPrint BSD_4_Clause = text "BSD4"
+    pPrint ISC = text "ISC"
+    pPrint CC_BY = text "CC-BY"
+    pPrint CC_BY_SA = text "CC-BY-SA"
+    pPrint CC_BY_ND = text "CC-BY-ND"
+    pPrint CC_BY_NC = text "CC-BY-NC"
+    pPrint CC_BY_NC_SA = text "CC-BY-NC-SA"
+    pPrint CC_BY_NC_ND = text "CC-BY-NC-ND"
+    pPrint CC0 = text "CC0"
+    pPrint CDDL = text "CDDL"
+    pPrint CPL = text "CPL"
+    pPrint EFL = text "EFL"
+    pPrint Expat = text "Expat"
+    pPrint GPL = text "GPL"
+    pPrint LGPL = text "LGPL"
+    pPrint GFDL = text "GFDL"
+    pPrint GFDL_NIV = text "GFDL-NIV"
+    pPrint LPPL = text "LPPL"
+    pPrint MPL = text "MPL"
+    pPrint Perl = text "Perl"
+    pPrint Python = text "Python"
+    pPrint QPL = text "QPL"
+    pPrint W3C = text "W3C"
+    pPrint Zlib = text "Zlib"
+    pPrint Zope = text "Zope"
+    pPrint (OtherLicense s) = text s
+
+-- | Convert the Cabal license to a Debian license.  I would welcome input
+-- on how to make this more correct.
+fromCabalLicense :: Cabal.License -> License
+fromCabalLicense x =
+    case x of
+      Cabal.GPL _ -> GPL -- FIXME - what about the version number?  same below
+      Cabal.AGPL _ -> OtherLicense (show x)
+      Cabal.LGPL _ -> LGPL
+      Cabal.BSD3 -> BSD_3_Clause
+      Cabal.BSD4 -> BSD_4_Clause
+      Cabal.MIT -> OtherLicense (show x)
+      Cabal.Apache _ -> Apache
+      Cabal.PublicDomain -> Public_Domain
+      Cabal.AllRightsReserved -> OtherLicense "AllRightsReserved"
+      Cabal.OtherLicense -> OtherLicense (show x)
+      Cabal.UnknownLicense _ -> OtherLicense (show x)
+#if MIN_VERSION_Cabal(1,20,0)
+      Cabal.MPL _ -> MPL
+#if MIN_VERSION_Cabal(1,22,0)
+      Cabal.BSD2 -> BSD_2_Clause
+      Cabal.ISC -> OtherLicense (show x)
+      Cabal.UnspecifiedLicense -> OtherLicense (show x)
+#endif
+#endif
+
+-- | Convert a Debian license to a Cabal license.  Additional cases
+-- and corrections welcome.
+toCabalLicense :: License -> Cabal.License
+toCabalLicense x =
+    -- This needs to be finished
+    case x of
+#if MIN_VERSION_Cabal(1,22,0)
+      BSD_2_Clause -> Cabal.BSD2
+#endif
+      BSD_3_Clause -> Cabal.BSD3
+      BSD_4_Clause -> Cabal.BSD4
+      OtherLicense s -> Cabal.UnknownLicense s
+      _ -> Cabal.UnknownLicense (show x)
+
+invalidLicense :: Text -> License
+invalidLicense = OtherLicense . unpack
+
+-- | I think we need an actual parser for license names.
+readLicense :: Text -> License
+readLicense t = let s = unpack (strip t) in fromMaybe (invalidLicense t) (readMaybe s)
